/*
 * Copyright (C) 2022 European Spallation Source ERIC.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package org.openepics.names.service;

import java.util.logging.Level;
import java.util.logging.Logger;

import org.apache.commons.lang3.StringUtils;
import org.openepics.names.service.exception.ServiceException;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

/**
 * Utility class to assist in handling of logs.
 *
 * @author Lars Johansson
 */
@Service
public class LogService {

    private static final String ORG_OPENEPICS_NAMES                     = "org.openepics.names";
    private static final String ORG_OPENEPICS_NAMES_UTIL_EXCEPTION_UTIL = "org.openepics.names.util.ExceptionUtil";

    private int logStackTraceLength;

    @Autowired
    protected LogService(@Value("${logutil.stacktrace.length:10}") int logStackTraceLength) {
        this.logStackTraceLength = logStackTraceLength;
    }

    /**
     * Log service exception.
     *
     * @param logger logger
     * @param level log level
     * @param e service exception
     */
    public void logServiceException(Logger logger, Level level, ServiceException e) {
        if (logger == null || level == null || e == null) {
            return;
        }

        String msg = !StringUtils.isEmpty(e.getMessage())
                ? e.getMessage()
                : "";
        String details = !StringUtils.isEmpty(e.getDetails())
                ? " ### " + e.getDetails()
                : "";
        String field = !StringUtils.isEmpty(e.getField())
                ? " ### " + e.getField()
                : "";
        msg = msg + details + field;

        logger.log(level, msg);
    }

    /**
     * Log stack trace elements.
     *
     * @param logger logger
     * @param level log level
     * @param e service exception
     */
    public void logStackTraceElements(Logger logger, Level level, ServiceException e) {
        logStackTraceElements(logger, level, e, logStackTraceLength, ORG_OPENEPICS_NAMES, ORG_OPENEPICS_NAMES_UTIL_EXCEPTION_UTIL);
    }

    /**
     * Log stack trace elements.
     *
     * @param logger logger
     * @param level log level
     * @param e exception
     */
    public void logStackTraceElements(Logger logger, Level level, Exception e) {
        logStackTraceElements(logger, level, e, logStackTraceLength, ORG_OPENEPICS_NAMES, ORG_OPENEPICS_NAMES_UTIL_EXCEPTION_UTIL);
    }

    /**
     * Log stack trace elements.
     *
     * @param logger logger
     * @param level log level
     * @param e service exception
     * @param maxNumberOfLogs max number of logs
     * @param filterInclude filter include
     * @param filterExclude filter exclude
     */
    public void logStackTraceElements(Logger logger, Level level, ServiceException e, int maxNumberOfLogs, String filterInclude, String filterExclude) {
        if (logger == null || level == null || e == null || maxNumberOfLogs <= 0) {
            return;
        }

        logStackTraceElements(logger, level, e.getStackTrace(), maxNumberOfLogs, filterInclude, filterExclude);
    }


    /**
     * Log stack trace elements.
     *
     * @param logger logger
     * @param level log level
     * @param e exception
     * @param maxNumberOfLogs max number of logs
     * @param filterInclude filter include
     * @param filterExclude filter exclude
     */
    public void logStackTraceElements(Logger logger, Level level, Exception e, int maxNumberOfLogs, String filterInclude, String filterExclude) {
        if (logger == null || level == null || e == null || maxNumberOfLogs <= 0) {
            return;
        }

        logStackTraceElements(logger, level, e.getStackTrace(), maxNumberOfLogs, filterInclude, filterExclude);
    }

    /**
     * Log stack trace elements.
     *
     * @param logger logger
     * @param level log level
     * @param stackTraceElements stack trace elements
     * @param maxNumberOfLogs max number of logs
     * @param filterInclude filter include
     * @param filterExclude filter exclude
     */
    private void logStackTraceElements(Logger logger, Level level, StackTraceElement[] stackTraceElements, int maxNumberOfLogs, String filterInclude, String filterExclude) {
        if (logger == null || level == null || stackTraceElements == null || maxNumberOfLogs <= 0) {
            return;
        }

        String str;
        int count = 0;
        for (StackTraceElement stackTraceElement : stackTraceElements) {
            str = stackTraceElement.toString();

            if  ((StringUtils.isEmpty(filterInclude) || str.contains(filterInclude))
                    && !(!StringUtils.isEmpty(filterExclude) && str.contains(filterExclude))) {
                count++;
                logger.log(level, str);
                if (count == maxNumberOfLogs) {
                    break;
                }
            }
        }
    }

}
